@@ -129,15 +129,13 @@ fill_sources_menu (GcalEditDialog *dialog)
     {
       ESource *source;
       GMenuItem *item;
-      ESourceSelectable *extension;
       GdkRGBA color;
       GdkPixbuf *pix;
 
       source = E_SOURCE (aux->data);
 
       /* retrieve color */
-      extension = E_SOURCE_SELECTABLE (e_source_get_extension (source, E_SOURCE_EXTENSION_CALENDAR));
-      gdk_rgba_parse (&color, e_source_selectable_get_color (E_SOURCE_SELECTABLE (extension)));
+      gdk_rgba_parse (&color, get_color_name_from_source (source));
       pix = gcal_get_pixbuf_from_color (&color, 16);;
 
       /* menu item */
@@ -194,12 +192,10 @@ on_calendar_selected (GtkWidget *menu_item,
       if (g_strcmp0 (e_source_get_uid (source), uid) == 0)
       {
         GdkRGBA color;
-        ESourceSelectable *extension;
         GdkPixbuf *pix;
 
         /* retrieve color */
-        extension = E_SOURCE_SELECTABLE (e_source_get_extension (source, E_SOURCE_EXTENSION_CALENDAR));
-        gdk_rgba_parse (&color, e_source_selectable_get_color (E_SOURCE_SELECTABLE (extension)));
+        gdk_rgba_parse (&color, get_color_name_from_source (source));
 
         pix = gcal_get_pixbuf_from_color (&color, 16);
         gtk_image_set_from_pixbuf (GTK_IMAGE (priv->source_image), pix);
@@ -426,7 +422,7 @@ update_time (GtkEntry   *entry,
                                        update_time,
                                        user_data);
       g_signal_handlers_unblock_by_func (priv->end_time_selector,
-                                       update_date,
+                                       update_time,
                                        user_data);
     }
 
@@ -764,7 +760,6 @@ gcal_edit_dialog_set_event_data (GcalEditDialog *dialog,
 
   GdkRGBA color;
   GdkPixbuf *pix;
-  ESourceSelectable *extension;
 
   const gchar *const_text = NULL;
   gboolean all_day;
@@ -825,10 +820,7 @@ gcal_edit_dialog_set_event_data (GcalEditDialog *dialog,
     gtk_entry_set_text (GTK_ENTRY (priv->summary_entry), e_summary.value);
 
   /* dialog titlebar's title & subtitle */
-  extension = E_SOURCE_SELECTABLE (e_source_get_extension (data->source, E_SOURCE_EXTENSION_CALENDAR));
-  gdk_rgba_parse (
-      &color,
-      e_source_selectable_get_color (E_SOURCE_SELECTABLE (extension)));
+  gdk_rgba_parse (&color, get_color_name_from_source (data->source));
 
   pix = gcal_get_pixbuf_from_color (&color, 16);
   gtk_image_set_from_pixbuf (GTK_IMAGE (priv->source_image), pix);
@@ -846,31 +838,37 @@ gcal_edit_dialog_set_event_data (GcalEditDialog *dialog,
 
   /* start date */
   gcal_date_selector_set_date (GCAL_DATE_SELECTOR (priv->start_date_selector),
-                            dtstart.value->day,
-                            dtstart.value->month,
-                            dtstart.value->year);
+                               dtstart.value->day,
+                               dtstart.value->month,
+                               dtstart.value->year);
 
   /* start time */
-  if (all_day)
+  if (!all_day)
     {
-      dtstart.value->hour = 0;
-      dtstart.value->minute = 0;
+      icaltimetype *date = gcal_dup_icaltime (dtstart.value);
+      *date = icaltime_convert_to_zone (*(dtstart.value), e_cal_util_get_system_timezone ());
+      gcal_time_selector_set_time (GCAL_TIME_SELECTOR (priv->start_time_selector), date->hour, date->minute);
+      g_free (date);
     }
-  gcal_time_selector_set_time (GCAL_TIME_SELECTOR (priv->start_time_selector), dtstart.value->hour, dtstart.value->minute);
 
   /* end date */
   if (dtend.value != NULL)
     {
       gcal_date_selector_set_date (GCAL_DATE_SELECTOR (priv->end_date_selector),
-                                dtend.value->day, dtend.value->month, dtend.value->year);
+                                   dtend.value->day, dtend.value->month, dtend.value->year);
 
       if (!all_day)
-        gcal_time_selector_set_time (GCAL_TIME_SELECTOR (priv->end_time_selector), dtend.value->hour, dtend.value->minute);
+        {
+          icaltimetype *date = gcal_dup_icaltime (dtend.value);
+          *date = icaltime_convert_to_zone (*(dtstart.value), e_cal_util_get_system_timezone ());
+          gcal_time_selector_set_time (GCAL_TIME_SELECTOR (priv->end_time_selector), date->hour, date->minute);
+          g_free (date);
+        }
     }
   else
     {
       gcal_date_selector_set_date (GCAL_DATE_SELECTOR (priv->end_date_selector),
-                                dtstart.value->day, dtstart.value->month, dtstart.value->year);
+                                   dtstart.value->day, dtstart.value->month, dtstart.value->year);
       gcal_time_selector_set_time (GCAL_TIME_SELECTOR (priv->end_time_selector), dtstart.value->hour, dtstart.value->minute);
     }
 
