@@ -49,10 +49,10 @@ struct _GcalYearViewPrivate
   GtkWidget    *sidebar;
   GtkWidget    *events_sidebar;
   GtkWidget    *navigator_stack;
+  GtkWidget    *no_events_title;
   GtkWidget    *navigator_sidebar;
 
   GtkWidget    *popover; /* Popover for popover_mode */
-  GtkWidget    *popover_sidebar;
 
   /* manager singleton */
   GcalManager  *manager;
@@ -68,7 +68,6 @@ struct _GcalYearViewPrivate
   gboolean      popover_mode;
   gboolean      button_pressed;
   ButtonData   *selected_data;
-  gboolean      update_sidebar_needed;
 
   /**
    * first day of the week according to user locale, being
@@ -120,34 +119,8 @@ update_date (GcalYearView *year_view,
 {
   GcalYearViewPrivate *priv = year_view->priv;
 
-  if (priv->date == NULL || priv->date->year != new_date->year)
-    {
-      time_t range_start, range_end;
-      icaltimetype date;
-      icaltimezone* default_zone = gcal_manager_get_system_timezone (priv->manager);
-
-      date = *new_date;
-      date.day = 1;
-      date.month = 1;
-      date.hour = 0;
-      date.minute = 0;
-      date.second = 0;
-      date.is_date = 0;
-      range_start = icaltime_as_timet_with_zone (date, default_zone);
-
-      date.day = 31;
-      date.month = 12;
-      date.hour = 23;
-      date.minute = 59;
-      date.second = 0;
-      range_end = icaltime_as_timet_with_zone (date, default_zone);
-
-      gcal_manager_set_subscriber (priv->manager, E_CAL_DATA_MODEL_SUBSCRIBER (year_view), range_start, range_end);
-      gtk_widget_queue_draw (GTK_WIDGET (year_view));
-
-      if (priv->start_selected_date->day != 0)
-        reset_sidebar (year_view);
-    }
+  if (priv->date != NULL && priv->date->year != new_date->year && priv->start_selected_date->day != 0)
+    reset_sidebar (year_view);
 
   if (priv->date != NULL)
     g_free (priv->date);
@@ -188,20 +161,10 @@ order_selected_data (ButtonData *selected_data)
 }
 
 static void
-update_sidebar (GcalYearView *year_view)
+update_selected_dates_from_button_data (GcalYearView *year_view)
 {
   GcalYearViewPrivate *priv = year_view->priv;
 
-  GtkWidget *child_widget;
-
-  GList *events, *l;
-  GList **days_widgets_array;
-
-  gint i, days_span;
-
-  const icaltimetype *dt_start, *dt_end;
-  icaltimetype date, second_date;
-
   if (priv->selected_data->start_day != 0)
     {
       ButtonData selected_data = *(priv->selected_data);
@@ -211,7 +174,7 @@ update_sidebar (GcalYearView *year_view)
       priv->start_selected_date->month = selected_data.start_month + 1;
       priv->start_selected_date->year = priv->date->year;
 
-      priv->end_selected_date->day = selected_data.end_day + 1;
+      priv->end_selected_date->day = selected_data.end_day;
       priv->end_selected_date->month = selected_data.end_month + 1;
       priv->end_selected_date->year = priv->date->year;
       priv->end_selected_date->hour = 23;
@@ -225,7 +188,6 @@ update_sidebar (GcalYearView *year_view)
       priv->start_selected_date->minute = 0;
 
       *(priv->end_selected_date) = *(priv->current_date);
-      priv->end_selected_date->day += 1;
       priv->end_selected_date->hour = 23;
       priv->end_selected_date->minute = 59;
       *(priv->end_selected_date) = icaltime_normalize (*(priv->end_selected_date));
@@ -237,88 +199,145 @@ update_sidebar (GcalYearView *year_view)
       priv->end_selected_date->month = 12;
       priv->end_selected_date->year = priv->start_selected_date->year;
     }
+}
+
+static void
+update_no_events_page (GcalYearView *year_view)
+{
+  GcalYearViewPrivate *priv;
+
+  gchar *title;
+  gboolean has_range;
+
+  priv = year_view->priv;
+  has_range = (priv->start_selected_date->day != priv->end_selected_date->day ||
+               priv->start_selected_date->month != priv->end_selected_date->month);
+
+  if (icaltime_compare_date (priv->current_date, priv->start_selected_date) == 0)
+    {
+      title = g_strdup_printf ("%s%s", _("Today"), has_range ? "…" : "");
+    }
+  else
+    {
+      title = g_strdup_printf ("%s %d%s",
+                               gcal_get_month_name (priv->start_selected_date->month - 1),
+                               priv->start_selected_date->day,
+                               has_range ? "…" : "");
+    }
+
+  gtk_label_set_text (GTK_LABEL (priv->no_events_title), title);
+  g_free (title);
+}
+
+static void
+add_event_to_day_array (GcalYearView  *year_view,
+                        GcalEventData *event_data,
+                        GList        **days_widgets_array,
+                        gint           days_span)
+{
+  GcalYearViewPrivate *priv;
+  GtkWidget *child_widget;
+
+  const icaltimetype *dt_start, *dt_end;
+  icaltimetype date, second_date;
+
+  gint i;
+  gboolean child_widget_used = FALSE;
+
+  priv = year_view->priv;
+  child_widget = gcal_event_widget_new_from_data (event_data);
+  gcal_event_widget_set_read_only (GCAL_EVENT_WIDGET (child_widget),
+                                   gcal_manager_is_client_writable (priv->manager, event_data->source));
+
+  dt_start = gcal_event_widget_peek_start_date (GCAL_EVENT_WIDGET (child_widget));
+  dt_end = gcal_event_widget_peek_end_date (GCAL_EVENT_WIDGET (child_widget));
+
+  /* normalize date on each new event */
+  date = *(priv->start_selected_date);
+  second_date = *(priv->start_selected_date);
+  second_date.hour = 23;
+  second_date.minute = 59;
+
+  /* marking and cloning */
+  for (i = 0; i < days_span; i++)
+    {
+      GtkWidget *cloned_child = child_widget;
+      gint start_comparison, end_comparison;
+
+      if (i != 0)
+        {
+          icaltime_adjust (&date, 1, 0, 0, 0);
+          icaltime_adjust (&second_date, 1, 0, 0, 0);
+        }
+
+      start_comparison = icaltime_compare_date (dt_start, &date);
+      if (start_comparison <= 0)
+        {
+          if (child_widget_used)
+            cloned_child = gcal_event_widget_clone (GCAL_EVENT_WIDGET (child_widget));
+          else
+            child_widget_used = TRUE;
+        }
+      else
+        {
+          cloned_child = NULL;
+        }
+
+      if (cloned_child != NULL)
+        {
+          days_widgets_array[i] = g_list_insert_sorted (days_widgets_array[i],
+                                                        cloned_child,
+                                                        (GCompareFunc) gcal_event_widget_compare_for_single_day);
+
+          end_comparison = icaltime_compare_date (&second_date, dt_end);
+          /* XXX: hack ensuring allday events with end_date a day after */
+          if (end_comparison == -1 && second_date.year == dt_end->year && dt_end->is_date == 1)
+            end_comparison = 0;
+
+          if (start_comparison < 0 && end_comparison < 0)
+            gtk_style_context_add_class (gtk_widget_get_style_context (cloned_child), "slanted");
+          else if (start_comparison < 0)
+            gtk_style_context_add_class (gtk_widget_get_style_context (cloned_child), "slanted-start");
+          else if (end_comparison < 0)
+            gtk_style_context_add_class (gtk_widget_get_style_context (cloned_child), "slanted-end");
+
+          if (end_comparison == 0)
+            break;
+        }
+    }
+}
+
+static void
+update_sidebar (GcalYearView *year_view)
+{
+  GcalYearViewPrivate *priv = year_view->priv;
+
+  GtkWidget *child_widget;
+  GList *events, *l;
+  GList **days_widgets_array;
+  gint i, days_span;
+
+  update_selected_dates_from_button_data (year_view);
 
   gtk_container_foreach (GTK_CONTAINER (priv->events_sidebar), (GtkCallback) gtk_widget_destroy, NULL);
 
-  days_span = icaltime_day_of_year(*(priv->end_selected_date)) - icaltime_day_of_year(*(priv->start_selected_date));
-  days_span = days_span == 0 ? 1 : days_span;
+  days_span = icaltime_day_of_year(*(priv->end_selected_date)) - icaltime_day_of_year(*(priv->start_selected_date)) + 1;
   days_widgets_array = g_new0 (GList*, days_span);
 
   events = gcal_manager_get_events (priv->manager, priv->start_selected_date, priv->end_selected_date);
-
-  /* XXX: compare_date_only does not work if is_date field is different */
-  priv->start_selected_date->is_date = priv->current_date->is_date;
-  if (events == NULL && icaltime_compare_date_only (*(priv->start_selected_date), *(priv->current_date)) == 0)
+  if (events == NULL)
     {
       days_span = 0;
+      update_no_events_page (year_view);
       gtk_stack_set_visible_child_name (GTK_STACK (priv->navigator_stack), "no-events");
     }
   else
     {
       gtk_stack_set_visible_child_name (GTK_STACK (priv->navigator_stack), "events-list");
     }
-  priv->start_selected_date->is_date = 0;
 
   for (l = events; l != NULL; l = g_list_next (l))
-    {
-      GcalEventData *data = l->data;
-      gboolean child_widget_used = FALSE;
-
-      child_widget = gcal_event_widget_new_from_data (data);
-      gcal_event_widget_set_read_only (GCAL_EVENT_WIDGET (child_widget),
-                                       gcal_manager_is_client_writable (priv->manager, data->source));
-
-      dt_start = gcal_event_widget_peek_start_date (GCAL_EVENT_WIDGET (child_widget));
-      dt_end = gcal_event_widget_peek_end_date (GCAL_EVENT_WIDGET (child_widget));
-
-      /* normalize date on each new event */
-      date = *(priv->start_selected_date);
-      second_date = *(priv->start_selected_date);
-      second_date.hour = 23;
-      second_date.minute = 59;
-
-      /* marking and cloning */
-      for (i = 0; i < days_span; i++)
-        {
-          GtkWidget *cloned_child = child_widget;
-          gint start_comparison, end_comparison;
-
-          if (i != 0)
-            {
-              icaltime_adjust (&date, 1, 0, 0, 0);
-              icaltime_adjust (&second_date, 1, 0, 0, 0);
-            }
-
-          start_comparison = icaltime_compare (*dt_start, date);
-          if (start_comparison == 0 || start_comparison == -1 || (date.day == dt_start->day && date.month == dt_start->month))
-            {
-              if (child_widget_used)
-                cloned_child = gcal_event_widget_clone (GCAL_EVENT_WIDGET (child_widget));
-              else
-                child_widget_used = TRUE;
-            }
-          else
-            {
-              cloned_child = NULL;
-            }
-
-          if (cloned_child != NULL)
-            {
-              days_widgets_array[i] = g_list_insert_sorted (days_widgets_array[i],
-                                                            cloned_child,
-                                                            (GCompareFunc) gcal_event_widget_compare_for_single_day);
-
-              if (start_comparison == -1)
-                gtk_style_context_add_class (gtk_widget_get_style_context (cloned_child), "slanted-start");
-
-              end_comparison = icaltime_compare (second_date, *dt_end);
-              if (end_comparison == -1)
-                gtk_style_context_add_class (gtk_widget_get_style_context (cloned_child), "slanted-end");
-              else
-                break;
-            }
-        }
-    }
+    add_event_to_day_array (year_view, l->data, days_widgets_array, days_span);
 
   for (i = 0; i < days_span; i++)
     {
@@ -335,8 +354,8 @@ update_sidebar (GcalYearView *year_view)
       g_list_free (current_day);
     }
 
-  g_list_free_full (events, g_free);
   g_free (days_widgets_array);
+  g_list_free_full (events, g_free);
 }
 
 static void
@@ -381,9 +400,7 @@ update_sidebar_headers (GtkListBoxRow *row,
       date = *(priv->start_selected_date);
       icaltime_adjust (&date, row_shift, 0, 0, 0);
 
-      /* XXX: compare_date_only does not work if is_date field is different */
-      date.is_date = priv->current_date->is_date;
-      if (icaltime_compare_date_only (date, *(priv->current_date)) == 0)
+      if (icaltime_compare_date (&date, priv->current_date) == 0)
         label_str = g_strdup (_("Today"));
       else
         label_str = g_strdup_printf ("%s %d", gcal_get_month_name (date.month  - 1), date.day);
@@ -422,10 +439,28 @@ update_sidebar_headers (GtkListBoxRow *row,
     }
 
   if (row_header != NULL)
-    {
-      gtk_widget_show_all (row_header);
-      gtk_list_box_row_set_header (row, row_header);
-    }
+    gtk_widget_show_all (row_header);
+  gtk_list_box_row_set_header (row, row_header);
+}
+
+static gint
+sidebar_sort_func (GtkListBoxRow *row1,
+                   GtkListBoxRow *row2,
+                   gpointer       user_data)
+{
+  GtkWidget *row1_child, *row2_child;
+  gint result, row1_shift, row2_shift;
+
+  row1_child = gtk_bin_get_child (GTK_BIN (row1));
+  row2_child = gtk_bin_get_child (GTK_BIN (row2));
+  row1_shift = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (row1_child), "shift"));
+  row2_shift = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (row2_child), "shift"));
+
+  result = row1_shift - row2_shift;
+  if (result == 0)
+    return gcal_event_widget_compare_for_single_day (GCAL_EVENT_WIDGET (row1_child), GCAL_EVENT_WIDGET (row2_child));
+
+  return result;
 }
 
 static gboolean
@@ -647,12 +682,13 @@ draw_month_grid (GcalYearView *year_view,
         }
       else if (column == sunday_idx)
         {
+          gtk_style_context_save (context);
           gtk_style_context_add_class (context, "sunday");
           gtk_render_layout (context, cr,
                              box_side * (column + 0.5 + priv->k) + x + sw * box_padding_start - priv->k * layout_width,
                              box_side * (row + 1) + y + box_padding_top,
                              layout);
-          gtk_style_context_remove_class (context, "sunday");
+          gtk_style_context_restore (context);
         }
       else
         {
@@ -818,6 +854,12 @@ navigator_button_release_cb (GcalYearView   *year_view,
   priv->button_pressed = FALSE;
   priv->selected_data->end_day = day;
   priv->selected_data->end_month = month;
+
+  /* update date and notify */
+  priv->date->day = day;
+  priv->date->month = month + 1;
+  g_object_notify (G_OBJECT (year_view), "active-date");
+
   gtk_widget_queue_draw (widget);
 
   if (priv->popover_mode)
@@ -882,6 +924,9 @@ add_event_clicked_cb (GcalYearView *year_view,
   GcalYearViewPrivate *priv = year_view->priv;
   icaltimetype *start_date, *end_date = NULL;
 
+  if (priv->popover_mode)
+    gtk_widget_hide (priv->popover);
+
   if (priv->start_selected_date->day == 0)
     {
       start_date = gcal_dup_icaltime (priv->current_date);
@@ -890,6 +935,8 @@ add_event_clicked_cb (GcalYearView *year_view,
     {
       start_date = gcal_dup_icaltime (priv->start_selected_date);
       end_date = gcal_dup_icaltime (priv->end_selected_date);
+      end_date->day += 1;
+      *end_date = icaltime_normalize (*end_date);
       end_date->is_date = 1;
     }
 
@@ -915,13 +962,12 @@ gcal_year_view_finalize (GObject *object)
   g_free (priv->navigator_grid);
   g_free (priv->selected_data);
 
+  g_free (priv->start_selected_date);
+  g_free (priv->end_selected_date);
+
   if (priv->date != NULL)
     g_free (priv->date);
 
-  /* FIXME: move into window */
-  if (priv->current_date != NULL)
-    g_free (priv->current_date);
-
   G_OBJECT_CLASS (gcal_year_view_parent_class)->finalize (object);
 }
 
@@ -1013,35 +1059,28 @@ gcal_year_view_size_allocate (GtkWidget     *widget,
   gtk_style_context_restore (context);
 
   priv->popover_mode = (alloc->width < NAVIGATOR_CELL_WIDTH * 4 + padding_left * 8 + SIDEBAR_PREFERRED_WIDTH);
-  if (gtk_widget_get_visible (priv->sidebar) == priv->popover_mode)
-    gtk_widget_set_visible (priv->sidebar, !priv->popover_mode);
-
   if (priv->popover_mode && !gtk_widget_is_ancestor (priv->events_sidebar, priv->popover))
     {
-      GtkWidget *parent = gtk_widget_get_parent (priv->events_sidebar);
-      g_object_ref (priv->events_sidebar);
+      g_object_ref (priv->sidebar);
 
-      gtk_container_remove (GTK_CONTAINER (parent), priv->events_sidebar);
-      gtk_container_add (GTK_CONTAINER (priv->popover_sidebar), priv->events_sidebar);
+      gtk_container_remove (GTK_CONTAINER (widget), priv->sidebar);
+      gtk_container_add (GTK_CONTAINER (priv->popover), priv->sidebar);
 
-      gtk_widget_show_all (priv->events_sidebar);
-      g_object_unref (priv->events_sidebar);
-      gtk_widget_destroy (parent);
+      g_object_unref (priv->sidebar);
 
+      gtk_widget_show_all (priv->sidebar);
       popover_closed_cb (GCAL_YEAR_VIEW (widget), GTK_POPOVER (priv->popover));
     }
   else if (!priv->popover_mode && gtk_widget_is_ancestor (priv->events_sidebar, priv->popover))
     {
-      GtkWidget *parent = gtk_widget_get_parent (priv->events_sidebar);
-      g_object_ref (priv->events_sidebar);
+      g_object_ref (priv->sidebar);
 
-      gtk_container_remove (GTK_CONTAINER (parent), priv->events_sidebar);
-      gtk_container_add (GTK_CONTAINER (priv->navigator_sidebar), priv->events_sidebar);
+      gtk_container_remove (GTK_CONTAINER (priv->popover), priv->sidebar);
+      gtk_box_pack_end (GTK_BOX (widget), priv->sidebar, FALSE, TRUE, 0);
 
-      gtk_widget_show_all (priv->events_sidebar);
-      g_object_unref (priv->events_sidebar);
-      gtk_widget_destroy (parent);
+      g_object_unref (priv->sidebar);
 
+      gtk_widget_show (priv->sidebar);
       g_signal_handlers_block_by_func (priv->popover, popover_closed_cb, widget);
       gtk_widget_hide (priv->popover);
       g_signal_handlers_unblock_by_func (priv->popover, popover_closed_cb, widget);
@@ -1073,7 +1112,7 @@ gcal_year_view_get_children_by_uuid (GcalView    *view,
   for (l = children; l != NULL; l = g_list_next (l))
     {
       GcalEventWidget *child_widget = GCAL_EVENT_WIDGET (gtk_bin_get_child (GTK_BIN (l->data)));
-      if (g_strcmp0 (uuid, gcal_event_widget_peek_uuid (child_widget)) == 0)
+      if (child_widget != NULL && g_strcmp0 (uuid, gcal_event_widget_peek_uuid (child_widget)) == 0)
         result = g_list_append (result, child_widget);
     }
   g_list_free (children);
@@ -1081,31 +1120,101 @@ gcal_year_view_get_children_by_uuid (GcalView    *view,
   return result;
 }
 
+static void
+gcal_year_view_component_added (ECalDataModelSubscriber *subscriber,
+                                ECalClient              *client,
+                                ECalComponent           *comp)
+{
+  GcalYearViewPrivate *priv;
+  GcalYearView *year_view = GCAL_YEAR_VIEW (subscriber);
+
+  GcalEventData *data;
+  GList **days_widgets_array;
+  GList *l;
+  gint i, days_span;
+
+  ECalComponentDateTime date;
+  time_t event_start, event_end, range_start, range_end;
+  icaltimezone *zone;
+
+  priv = year_view->priv;
+  update_selected_dates_from_button_data (year_view);
+  days_span = icaltime_day_of_year(*(priv->end_selected_date)) - icaltime_day_of_year(*(priv->start_selected_date)) + 1;
+  days_widgets_array = g_new0 (GList*, days_span);
+
+  data = g_new0 (GcalEventData, 1);
+  data->source = e_client_get_source (E_CLIENT (client));
+  data->event_component = e_cal_component_clone (comp);
+
+  /* check if event belongs to range */
+  zone = gcal_manager_get_system_timezone (priv->manager);
+  range_start = icaltime_as_timet_with_zone (*(priv->start_selected_date), zone);
+  range_end = icaltime_as_timet_with_zone (*(priv->end_selected_date), zone);
+
+  e_cal_component_get_dtstart (comp, &date);
+  event_start = icaltime_as_timet_with_zone (*(date.value), date.value->zone != NULL ? date.value->zone : zone);
+  e_cal_component_free_datetime (&date);
+
+  e_cal_component_get_dtend (comp, &date);
+  if (date.value != NULL)
+    event_end = icaltime_as_timet_with_zone (*(date.value), date.value->zone != NULL ? date.value->zone : zone);
+  else
+    event_end = event_start;
+  e_cal_component_free_datetime (&date);
+
+  if (!((event_start <= range_start && event_end >= range_end) ||
+        (event_start >= range_start && event_end <= range_end) ||
+        (event_start >= range_start && event_start <= range_end) ||
+        (event_end >= range_start && event_end <= range_end)))
+    {
+      g_object_unref (data->event_component);
+      goto out;
+    }
+
+  add_event_to_day_array (year_view, data, days_widgets_array, days_span);
+  gtk_stack_set_visible_child_name (GTK_STACK (priv->navigator_stack), "events-list");
+
+  for (i = 0; i < days_span; i++)
+    {
+      GList *current_day = days_widgets_array[i];
+      for (l = current_day; l != NULL; l = g_list_next (l))
+        {
+          GtkWidget *child_widget = l->data;
+          gtk_widget_show (child_widget);
+          g_signal_connect (child_widget, "activate", G_CALLBACK (event_activated), year_view);
+          g_object_set_data (G_OBJECT (child_widget), "shift", GINT_TO_POINTER (i));
+          gtk_container_add (GTK_CONTAINER (priv->events_sidebar), child_widget);
+        }
+
+      g_list_free (current_day);
+    }
+
+out:
+  g_free (data);
+  g_free (days_widgets_array);
+}
+
 static void
 gcal_year_view_component_changed (ECalDataModelSubscriber *subscriber,
                                   ECalClient              *client,
                                   ECalComponent           *comp)
 {
   GcalYearViewPrivate *priv = GCAL_YEAR_VIEW (subscriber)->priv;
-  ECalComponentDateTime dtstart, dtend;
+  GList *children, *l;
+  gchar *uuid;
 
-  if (priv->update_sidebar_needed)
-    return;
+  uuid = get_uuid_from_component (e_client_get_source (E_CLIENT (client)), comp);
+  children = gtk_container_get_children (GTK_CONTAINER (priv->events_sidebar));
+  for (l = children; l != NULL; l = g_list_next (l))
+    {
+      GcalEventWidget *child_widget = GCAL_EVENT_WIDGET (gtk_bin_get_child (GTK_BIN (l->data)));
+      if (child_widget != NULL && g_strcmp0 (uuid, gcal_event_widget_peek_uuid (child_widget)) == 0)
+        gtk_widget_destroy (GTK_WIDGET (l->data));
+    }
+  g_list_free (children);
+  g_free (uuid);
 
-  e_cal_component_get_dtstart (comp, &dtstart);
-  e_cal_component_get_dtend (comp, &dtend);
-
-  /* FIXME: this implementation clear the sidebar are rebuild it
-   * it should only add, what's new, and add it sorted in its position */
-  /* XXX: implement using shift as data and GtkListBox sort_func */
-  /* XXX: comparing against zero is not reliable, but it can only result in TRUE */
-  if (icaltime_compare (*(dtstart.value), *(priv->start_selected_date)) >= 0)
-    priv->update_sidebar_needed = TRUE;
-  else if (icaltime_compare (*(dtend.value), *(priv->end_selected_date)) <= 0)
-    priv->update_sidebar_needed = TRUE;
-
-  e_cal_component_free_datetime (&dtstart);
-  e_cal_component_free_datetime (&dtend);
+  gcal_year_view_component_added (subscriber, client, comp);
 }
 
 static void
@@ -1118,6 +1227,7 @@ gcal_year_view_component_removed (ECalDataModelSubscriber *subscriber,
   GList *children, *l;
   ESource *source;
   gchar *uuid;
+  gboolean update_sidebar_needed = FALSE;
 
   source = e_client_get_source (E_CLIENT (client));
   if (rid != NULL)
@@ -1130,10 +1240,20 @@ gcal_year_view_component_removed (ECalDataModelSubscriber *subscriber,
     {
       GcalEventWidget *child_widget = GCAL_EVENT_WIDGET (gtk_bin_get_child (GTK_BIN (l->data)));
       if (child_widget != NULL && g_strcmp0 (uuid, gcal_event_widget_peek_uuid (child_widget)) == 0)
-        gtk_widget_destroy (GTK_WIDGET (child_widget));
+        {
+          if (g_list_length (children) == 1)
+            update_sidebar_needed = TRUE;
+          gtk_widget_destroy (GTK_WIDGET (l->data));
+        }
     }
   g_list_free (children);
   g_free (uuid);
+
+  if (update_sidebar_needed)
+    {
+      update_no_events_page (GCAL_YEAR_VIEW (subscriber));
+      gtk_stack_set_visible_child_name (GTK_STACK (priv->navigator_stack), "no-events");
+    }
 }
 
 static void
@@ -1145,12 +1265,6 @@ gcal_year_view_freeze (ECalDataModelSubscriber *subscriber)
 static void
 gcal_year_view_thaw (ECalDataModelSubscriber *subscriber)
 {
-  GcalYearViewPrivate *priv = GCAL_YEAR_VIEW (subscriber)->priv;
-  if (priv->update_sidebar_needed)
-    {
-      priv->update_sidebar_needed = FALSE;
-      update_sidebar (GCAL_YEAR_VIEW (subscriber));
-    }
 }
 
 static void
@@ -1183,8 +1297,8 @@ gcal_year_view_class_init (GcalYearViewClass *klass)
   gtk_widget_class_bind_template_child_private (widget_class, GcalYearView, events_sidebar);
   gtk_widget_class_bind_template_child_private (widget_class, GcalYearView, navigator_stack);
   gtk_widget_class_bind_template_child_private (widget_class, GcalYearView, navigator_sidebar);
+  gtk_widget_class_bind_template_child_private (widget_class, GcalYearView, no_events_title);
   gtk_widget_class_bind_template_child_private (widget_class, GcalYearView, popover);
-  gtk_widget_class_bind_template_child_private (widget_class, GcalYearView, popover_sidebar);
 
   gtk_widget_class_bind_template_callback (widget_class, draw_navigator);
   gtk_widget_class_bind_template_callback (widget_class, navigator_button_press_cb);
@@ -1214,6 +1328,7 @@ gcal_year_view_init (GcalYearView *self)
   self->priv->end_selected_date = g_new0 (icaltimetype, 1);
 
   gtk_list_box_set_header_func (GTK_LIST_BOX (self->priv->events_sidebar), update_sidebar_headers, self, NULL);
+  gtk_list_box_set_sort_func (GTK_LIST_BOX (self->priv->events_sidebar), sidebar_sort_func, NULL, NULL);
 }
 
 static void
@@ -1226,7 +1341,7 @@ gcal_view_interface_init (GcalViewIface *iface)
 static void
 gcal_data_model_subscriber_interface_init (ECalDataModelSubscriberInterface *iface)
 {
-  iface->component_added = gcal_year_view_component_changed;
+  iface->component_added = gcal_year_view_component_added;
   iface->component_modified = gcal_year_view_component_changed;
   iface->component_removed = gcal_year_view_component_removed;
   iface->freeze = gcal_year_view_freeze;
@@ -1234,12 +1349,6 @@ gcal_data_model_subscriber_interface_init (ECalDataModelSubscriberInterface *ifa
 }
 
 /* Public API */
-GcalYearView *
-gcal_year_view_new (void)
-{
-  return g_object_new (GCAL_TYPE_YEAR_VIEW, NULL);
-}
-
 void
 gcal_year_view_set_manager (GcalYearView *year_view,
                             GcalManager  *manager)
@@ -1265,13 +1374,9 @@ void
 gcal_year_view_set_current_date (GcalYearView *year_view,
                                  icaltimetype *current_date)
 {
-  /* FIXME: move into window */
-  year_view->priv->current_date = g_new0 (icaltimetype, 1);
-  *(year_view->priv->current_date) =
-  icaltime_current_time_with_zone (gcal_manager_get_system_timezone (year_view->priv->manager));
-  *(year_view->priv->current_date) = icaltime_set_timezone (year_view->priv->current_date,
-                                                            gcal_manager_get_system_timezone (year_view->priv->manager));
+  year_view->priv->current_date = current_date;
 
   /* Launches update */
+  gtk_widget_queue_draw (GTK_WIDGET (year_view));
   update_sidebar (year_view);
 }
